""" 
@project: excelfilling
@file: cli 
@time: 2021/09/28
@software: PyCharm
@author: zy7y
命令行扩展
"""
from pathlib import Path
from typing import Optional

from typer import Typer
from typer import echo
from typer import launch
from typer import confirm

from .src import main


app = Typer(help="- Excel表格模板数据填充工具 -")


@app.command("docs")
def visit_docs():
    """
    访问tablefill文档
    :return:
    """
    visit = confirm("即将访问tablefill文档，确认?")
    if visit:
        launch("https://gitee.com/zy7y/tablefill.git")


@app.command()
def files(path: Path, suffix: Optional[str] = ".json"):
    """
    列出绝对路径下所有 后缀为 xx 文件的绝对路径
    :param path: 需要查找的目录绝对路径
    :param suffix: 查找文件的后缀
    :return:

    >>> file_list(Path(__file__).absolute().parent)

    """
    p = Path(path)
    for child in p.glob(f"**/*{suffix}"):
        echo(child.absolute())


@app.command()
def generate(
    config: str,
    template: str,
    file: str,
    num: Optional[int] = 10,
    index: Optional[int] = 0,
):
    """
    读取行配置文件填充 excel 文件
    :param config: 列配置文件
    :param template: 待填充的excel模板
    :param file: 填充后的存储位置
    :param num: 填充总行数 默认 10条
    :param index: 填充到指定sheet页 默认 0 即 第一个
    :return:
    """
    main(config, template, file, num, index)


if __name__ == "__main__":
    app()



""" 
@project: excelfilling
@file: src 
@time: 2021/09/28
@software: PyCharm
@author: zy7y
功能实现
"""
import json
import time
from concurrent.futures import ThreadPoolExecutor

import xlrd
from faker import Faker
from rich.console import Console
from rich.progress import track
from xlutils.copy import copy
from typing import Dict
from queue import Queue

fake = Faker("zh-Cn")
console = Console()


def write_excel(new_worksheet, que: Queue, rows_exists):
    """实际写excel 的动作"""
    for index, value in enumerate(que.get()):
        new_worksheet.write(rows_exists + que.qsize(), index, value)


def create_file(score: str, path: str, que: Queue, sheet_index: int = 0):
    """
    多线程追加写入excel
    :param score: 源文件地址
    :param que: 需写入数据的队列
    :param path: 追加后的地址
    :param sheet_index: 追加数据到sheet页某个索引
    :return:
    """
    workbook = xlrd.open_workbook(score)
    sheets_name = workbook.sheet_names()
    worksheet = workbook.sheet_by_name(sheets_name[sheet_index])
    rows_exists = worksheet.nrows
    new_workbook = copy(workbook)
    new_worksheet = new_workbook.get_sheet(sheet_index)

    # 线程池
    with ThreadPoolExecutor() as executor:
        for _ in track(range(que.qsize()), description='写入...'):
            future = executor.submit(write_excel, new_worksheet, que, rows_exists)
            # todo 如不加操作会丢失数据
            future.result()

    new_workbook.save(path)


def row_data(row_cfg: Dict[str, str]) -> str:
    """
    根据列配置生成单个列数据
    :param row_cfg: 列配置
    :return: 虚拟数据
    """
    if row_cfg.get("type", "faker") == "faker":
        if parameter := row_cfg.get("var"):
            result = getattr(fake, row_cfg["func"])(**parameter)
        else:
            result = getattr(fake, row_cfg["func"])()
    elif row_cfg["type"] == "input":
        result = row_cfg.get("var", "")
    else:
        raise TypeError("type field must faker or input.")

    if not isinstance(result, str):
        result = str(result)

    if (var_first := row_cfg.get("varFirst")) is not None:
        result = str(var_first) + result

    if (var_end := row_cfg.get("varEnd")) is not None:
        result += str(var_end)
    return result


def generate_rows(cfg_path: str, que: Queue, number: int):
    """
    生成行数据
    :param cfg_path: 配置文件
    :param que: 队列
    :param number: 生成数量
    :return:
    """
    with open(cfg_path, encoding="utf-8") as f:
        conf = json.loads(f.read())

    [que.put([row_data(row_cfg) for row_cfg in conf]) for _ in track(range(number), description='生成...', total=number)]


def main(
        conf_path: str,
        score_file: str,
        generate_file: str,
        number: int = 10,
        sheet_index: int = 0,
):
    """
    主程序入口
    :param conf_path: 配置文件路径
    :param score_file: 源excel文件
    :param generate_file: 生成的文件
    :param number: 生成数据量
    :param sheet_index: 写在第几个sheet页
    :return:
    """
    console.print("欢迎使用tablefill 任务开始~")
    start = time.time()
    queue = Queue()
    generate_rows(conf_path, queue, number)
    create_file(score_file, generate_file, queue, sheet_index)
    console.print(f"您耗费了{round(time.time() - start, 2)}s, 为您向 {generate_file} 中填充了{number}条数据, 快去看看吧")


__all__ = [
    "main"
]

if __name__ == "__main__":
    main("../examples/demo.json", "../examples/demo.xlsx", "../examples/tablefill.xls")


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tablefill']

package_data = \
{'': ['*']}

install_requires = \
['Faker>=8.14.0,<9.0.0',
 'rich>=10.16.2,<11.0.0',
 'typer>=0.4.0,<0.5.0',
 'xlrd==1.2.0',
 'xlutils==2.0.0']

entry_points = \
{'console_scripts': ['fill = tablefill.cli:app']}

setup_kwargs = {
    'name': 'tablefill',
    'version': '0.2.0',
    'description': 'Excel模板数据填充,快速应对Web项目数据导入',
    'long_description': '# 背景\n在测试Web后台管理系统项目时，导入数据是个高频出现的功能，[tablefill](https://github.com/zy7y/tablefill)主要完成根据配置文件对模板进行填充数据\n\n# 使用\n**安装**\n```shell\npip install tablefill\n```\n**配置列数据类型**\n```json5\n[\n  {\n    "type": "faker", // 可选值 faker(默认值,可不写type这个字段)、input 会直接读取var 的值 由自己设置\n    "func": "name", // 对应的是 Faker 生成虚拟数据的那些方法名 https://faker.readthedocs.io/en/master/providers.html\n    "var": null, // 没有参数时可以不写该字段, 当type 为faker时 这部分会被作为func 对应函数名的入参\n    "varFirst": "前", // 如果不需要可以不写该字段, 会在 var 这个 参数 前面 加上 内容\n    "varEnd": "后" // 如果不需要可以不写该字段, 会在 var 这个 参数 后面 加上 内容\n  }\n]\n```\n**api参考[faker](https://faker.readthedocs.io/en/stable/providers.html)**\nphone_number: 生成手机号\nrandom_element: 列表中随机元素\nname: 随机名称\nssn: 身份证号\ndate: 随机日期\n\n\n*示例*\n```json\n[\n  {\n    "type": "input",\n    "var": "这列我输入"\n  },\n  {\n    "func": "phone_number"\n  },\n  {\n    "func": "random_int",\n    "var": {\n      "min": 10,\n      "max": 21\n    },\n    "varFirst": "编号",\n    "varEnd": "班"\n  },\n  {\n    "func": "random_element",\n    "var": {\n      "elements": ["小学", "高中", "初中"],\n    }\n  }\n]\n```\n**导入模板文件**\n> 需要是xlsx/xls文件\n[![4h3G3F.md.png](https://z3.ax1x.com/2021/09/29/4h3G3F.md.png)](https://imgtu.com/i/4h3G3F)\n\n**执行命令**\n```shell\n# --num 可选参数 默认 10条 ，这里就是30条\nfill generate 配置文件 模板文件 生成文件名 --num 30 \n\nfill generate "E:\\coding\\tablefill\\examples\\demo.json" "E:\\coding\\tablefill\\examples\\demo.xlsx"  demo.xls\n```\n\n**填充数据后的文件**\n[![4h8FbR.md.png](https://z3.ax1x.com/2021/09/29/4h8FbR.md.png)](https://imgtu.com/i/4h8FbR)\n\n**help**\n```shell\nfill --help\n```\n\n',
    'author': '柒意',
    'author_email': '396667207@qq.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitee.com/zy7y/tablefill',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
